@@ -14,7 +14,7 @@
 /**
  * AdwPreferencesPage:
  *
- * A page from [class@PreferencesWindow].
+ * A page from [class@PreferencesDialog].
  *
  * <picture>
  *   <source srcset="preferences-page-dark.png" media="(prefers-color-scheme: dark)">
@@ -35,6 +35,7 @@
 
 typedef struct
 {
+  GtkWidget *banner;
   GtkBox *box;
   GtkLabel *description;
   GtkWidget *scrolled_window;
@@ -63,6 +64,8 @@ enum {
   PROP_DESCRIPTION,
   PROP_NAME,
   PROP_USE_UNDERLINE,
+  PROP_DESCRIPTION_CENTERED,
+  PROP_BANNER,
   LAST_PROP,
 };
 
@@ -100,6 +103,12 @@ adw_preferences_page_get_property (GObject    *object,
   case PROP_USE_UNDERLINE:
     g_value_set_boolean (value, adw_preferences_page_get_use_underline (self));
     break;
+  case PROP_DESCRIPTION_CENTERED:
+    g_value_set_boolean (value, adw_preferences_page_get_description_centered (self));
+    break;
+  case PROP_BANNER:
+    g_value_set_object (value, adw_preferences_page_get_banner (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -129,6 +138,12 @@ adw_preferences_page_set_property (GObject      *object,
   case PROP_USE_UNDERLINE:
     adw_preferences_page_set_use_underline (self, g_value_get_boolean (value));
     break;
+  case PROP_DESCRIPTION_CENTERED:
+    adw_preferences_page_set_description_centered (self, g_value_get_boolean (value));
+    break;
+  case PROP_BANNER:
+    adw_preferences_page_set_banner (self, g_value_get_object (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -137,8 +152,13 @@ adw_preferences_page_set_property (GObject      *object,
 static void
 adw_preferences_page_dispose (GObject *object)
 {
+  AdwPreferencesPage *self = ADW_PREFERENCES_PAGE (object);
+  AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
+
   gtk_widget_dispose_template (GTK_WIDGET (object), ADW_TYPE_PREFERENCES_PAGE);
 
+  g_clear_pointer (&priv->banner, gtk_widget_unparent);
+
   G_OBJECT_CLASS (adw_preferences_page_parent_class)->dispose (object);
 }
 
@@ -170,7 +190,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
   widget_class->focus = adw_widget_focus_child;
 
   /**
-   * AdwPreferencesPage:icon-name: (attributes org.gtk.Property.get=adw_preferences_page_get_icon_name org.gtk.Property.set=adw_preferences_page_set_icon_name)
+   * AdwPreferencesPage:icon-name:
    *
    * The icon name for this page.
    */
@@ -180,7 +200,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesPage:title: (attributes org.gtk.Property.get=adw_preferences_page_get_title org.gtk.Property.set=adw_preferences_page_set_title)
+   * AdwPreferencesPage:title:
    *
    * The title for this page.
    */
@@ -190,10 +210,10 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesPage:description: (attributes org.gtk.Property.get=adw_preferences_page_get_description org.gtk.Property.set=adw_preferences_page_set_description)
+   * AdwPreferencesPage:description:
    *
    * The description to be displayed at the top of the page.
-   * 
+   *
    * Since: 1.4
    */
   props[PROP_DESCRIPTION] =
@@ -202,7 +222,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesPage:name: (attributes org.gtk.Property.get=adw_preferences_page_get_name org.gtk.Property.set=adw_preferences_page_set_name)
+   * AdwPreferencesPage:name:
    *
    * The name of this page.
    */
@@ -212,7 +232,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesPage:use-underline: (attributes org.gtk.Property.get=adw_preferences_page_get_use_underline org.gtk.Property.set=adw_preferences_page_set_use_underline)
+   * AdwPreferencesPage:use-underline:
    *
    * Whether an embedded underline in the title indicates a mnemonic.
    */
@@ -221,6 +241,30 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwPreferencesPage:description-centered:
+   *
+   * Whether the description should be centered.
+   *
+   * Since: 1.6
+   */
+  props[PROP_DESCRIPTION_CENTERED] =
+    g_param_spec_boolean ("description-centered", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwPreferencesPage:banner:
+   *
+   * A [class@Banner] displayed at the top of the page.
+   *
+   * Since: 1.7
+   */
+  props[PROP_BANNER] =
+    g_param_spec_object ("banner", NULL, NULL,
+                         ADW_TYPE_BANNER,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -231,13 +275,16 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
 
   gtk_widget_class_set_css_name (widget_class, "preferencespage");
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
-  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
+  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
 }
 
 static void
 adw_preferences_page_init (AdwPreferencesPage *self)
 {
   AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
+  GtkLayoutManager *layout = gtk_widget_get_layout_manager (GTK_WIDGET (self));
+
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (layout), GTK_ORIENTATION_VERTICAL);
 
   priv->title = g_strdup ("");
 
@@ -253,10 +300,15 @@ adw_preferences_page_buildable_add_child (GtkBuildable *buildable,
   AdwPreferencesPage *self = ADW_PREFERENCES_PAGE (buildable);
   AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
 
-  if (priv->box && ADW_IS_PREFERENCES_GROUP (child))
+  if (priv->box && ADW_IS_PREFERENCES_GROUP (child)) {
     adw_preferences_page_add (self, ADW_PREFERENCES_GROUP (child));
-  else
+  } else if (priv->scrolled_window && GTK_IS_WIDGET (child)) {
+    g_critical ("Trying to add %s as a child to an AdwPreferencePage, "
+                "but only AdwPreferencesPage is allowed",
+                G_OBJECT_TYPE_NAME (child));
+  } else {
     parent_buildable_iface->add_child (buildable, builder, child, type);
+  }
 }
 
 static void
@@ -325,7 +377,7 @@ adw_preferences_page_remove (AdwPreferencesPage  *self,
 }
 
 /**
- * adw_preferences_page_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_preferences_page_get_icon_name:
  * @self: a preferences page
  *
  * Gets the icon name for @self.
@@ -345,7 +397,7 @@ adw_preferences_page_get_icon_name (AdwPreferencesPage *self)
 }
 
 /**
- * adw_preferences_page_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_preferences_page_set_icon_name:
  * @self: a preferences page
  * @icon_name: (nullable): the icon name
  *
@@ -368,7 +420,7 @@ adw_preferences_page_set_icon_name (AdwPreferencesPage *self,
 }
 
 /**
- * adw_preferences_page_get_title: (attributes org.gtk.Method.get_property=title)
+ * adw_preferences_page_get_title:
  * @self: a preferences page
  *
  * Gets the title of @self.
@@ -388,7 +440,7 @@ adw_preferences_page_get_title (AdwPreferencesPage *self)
 }
 
 /**
- * adw_preferences_page_set_title: (attributes org.gtk.Method.set_property=title)
+ * adw_preferences_page_set_title:
  * @self: a preferences page
  * @title: the title
  *
@@ -411,13 +463,13 @@ adw_preferences_page_set_title (AdwPreferencesPage *self,
 }
 
 /**
- * adw_preferences_page_get_description: (attributes org.gtk.Method.get_property=description)
+ * adw_preferences_page_get_description:
  * @self: a preferences page
  *
  * Gets the description of @self.
  *
  * Returns: the description of @self.
- * 
+ *
  * Since: 1.4
  */
 const char *
@@ -429,18 +481,18 @@ adw_preferences_page_get_description (AdwPreferencesPage *self)
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  return gtk_label_get_text (priv->description);
+  return gtk_label_get_label (priv->description);
 }
 
 /**
- * adw_preferences_page_set_description: (attributes org.gtk.Method.set_property=description)
+ * adw_preferences_page_set_description:
  * @self: a preferences page
  * @description: the description
  *
  * Sets the description of @self.
- * 
+ *
  * The description is displayed at the top of the page.
- * 
+ *
  * Since: 1.4
  */
 void
@@ -464,7 +516,7 @@ adw_preferences_page_set_description (AdwPreferencesPage *self,
 }
 
 /**
- * adw_preferences_page_get_name: (attributes org.gtk.Method.get_property=name)
+ * adw_preferences_page_get_name:
  * @self: a preferences page
  *
  * Gets the name of @self.
@@ -484,7 +536,7 @@ adw_preferences_page_get_name (AdwPreferencesPage *self)
 }
 
 /**
- * adw_preferences_page_set_name: (attributes org.gtk.Method.set_property=name)
+ * adw_preferences_page_set_name:
  * @self: a preferences page
  * @name: (nullable): the name
  *
@@ -507,7 +559,7 @@ adw_preferences_page_set_name (AdwPreferencesPage *self,
 }
 
 /**
- * adw_preferences_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * adw_preferences_page_get_use_underline:
  * @self: a preferences page
  *
  * Gets whether an embedded underline in the title indicates a mnemonic.
@@ -527,7 +579,7 @@ adw_preferences_page_get_use_underline (AdwPreferencesPage *self)
 }
 
 /**
- * adw_preferences_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * adw_preferences_page_set_use_underline:
  * @self: a preferences page
  * @use_underline: `TRUE` if underlines in the text indicate mnemonics
  *
@@ -553,6 +605,121 @@ adw_preferences_page_set_use_underline (AdwPreferencesPage *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_UNDERLINE]);
 }
 
+/**
+ * adw_preferences_page_get_description_centered:
+ * @self: a preferences page
+ *
+ * Gets whether the description is centered.
+ *
+ * Returns: whether the description is centered.
+ *
+ * Since: 1.6
+ */
+gboolean
+adw_preferences_page_get_description_centered (AdwPreferencesPage *self)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_PAGE (self), FALSE);
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  return gtk_label_get_justify (priv->description) == GTK_JUSTIFY_CENTER;
+}
+
+/**
+ * adw_preferences_page_set_description_centered:
+ * @self: a preferences page
+ * @centered: If the description should be centered
+ *
+ * Sets whether the description should be centered.
+ *
+ * Since: 1.6
+ */
+void
+adw_preferences_page_set_description_centered (AdwPreferencesPage *self,
+                                               gboolean            centered)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (self));
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  centered = !!centered;
+
+  if (adw_preferences_page_get_description_centered (self) == centered)
+    return;
+
+  if (centered) {
+    gtk_label_set_justify (priv->description, GTK_JUSTIFY_CENTER);
+    gtk_label_set_xalign (priv->description, 0.5);
+  } else {
+    gtk_label_set_justify (priv->description, GTK_JUSTIFY_LEFT);
+    gtk_label_set_xalign (priv->description, 0);
+  }
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DESCRIPTION_CENTERED]);
+}
+
+/**
+ * adw_preferences_page_get_banner:
+ * @self: a preferences page
+ *
+ * Gets the banner displayed at the top of the page.
+ *
+ * Returns: (nullable) (transfer none): the banner for @self
+ *
+ * Since: 1.7
+ */
+AdwBanner *
+adw_preferences_page_get_banner (AdwPreferencesPage *self)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_PAGE (self), NULL);
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  return ADW_BANNER (priv->banner);
+}
+
+/**
+ * adw_preferences_page_set_banner:
+ * @self: a preferences page
+ * @banner: (nullable): the banner to display at the top of the page
+ *
+ * Sets the banner displayed at the top of the page.
+ *
+ * Since: 1.7
+ */
+void
+adw_preferences_page_set_banner (AdwPreferencesPage *self,
+                                 AdwBanner          *banner)
+{
+  AdwPreferencesPagePrivate *priv;
+  GtkWidget *banner_widget;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (self));
+  g_return_if_fail (banner == NULL || ADW_IS_BANNER (banner));
+
+  priv = adw_preferences_page_get_instance_private (self);
+  banner_widget = GTK_WIDGET (banner);
+
+  if (priv->banner == banner_widget)
+    return;
+
+  if (priv->banner)
+    gtk_widget_unparent (priv->banner);
+
+  priv->banner = banner_widget;
+
+  if (priv->banner)
+    gtk_widget_insert_after (priv->banner, GTK_WIDGET (self), NULL);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_BANNER]);
+}
+
 static GListModel *
 preferences_group_to_rows (AdwPreferencesGroup *group)
 {
