@@ -6,15 +6,12 @@
 
 #include <adwaita.h>
 
-int activated;
-
 static void
-activated_cb (GtkWidget *widget, gpointer data)
+increment (int *data)
 {
-  activated++;
+  (*data)++;
 }
 
-
 static void
 test_adw_action_row_add_remove (void)
 {
@@ -43,16 +40,27 @@ static void
 test_adw_action_row_subtitle (void)
 {
   AdwActionRow *row = g_object_ref_sink (ADW_ACTION_ROW (adw_action_row_new ()));
+  int notified = 0;
+
   g_assert_nonnull (row);
 
+  g_signal_connect_swapped (row, "notify::subtitle", G_CALLBACK (increment), &notified);
+
   g_assert_cmpstr (adw_action_row_get_subtitle (row), ==, "");
+  g_assert_cmpint (notified, ==, 0);
 
   adw_action_row_set_subtitle (row, "Dummy subtitle");
   g_assert_cmpstr (adw_action_row_get_subtitle (row), ==, "Dummy subtitle");
+  g_assert_cmpint (notified, ==, 1);
+
+  adw_action_row_set_subtitle (row, "<b>Dummy subtitle</b>");
+  g_assert_cmpstr (adw_action_row_get_subtitle (row), ==, "<b>Dummy subtitle</b>");
+  g_assert_cmpint (notified, ==, 2);
 
   adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (row), FALSE);
   adw_action_row_set_subtitle (row, "Invalid <b>markup");
   g_assert_cmpstr (adw_action_row_get_subtitle (row), ==, "Invalid <b>markup");
+  g_assert_cmpint (notified, ==, 3);
 
   g_assert_finalize_object (row);
 }
@@ -121,10 +129,11 @@ static void
 test_adw_action_row_activate (void)
 {
   AdwActionRow *row = g_object_ref_sink (ADW_ACTION_ROW (adw_action_row_new ()));
+  int activated = 0;
+
   g_assert_nonnull (row);
 
-  activated = 0;
-  g_signal_connect (row, "activated", G_CALLBACK (activated_cb), NULL);
+  g_signal_connect_swapped (row, "activated", G_CALLBACK (increment), &activated);
 
   adw_action_row_activate (row);
   g_assert_cmpint (activated, ==, 1);
